<?php
# It is the context of the subscription, in which a customer can manage its VPSes.
# It must correspond to a tenant created for the subscriber in the remote application system.
require "aps/2/runtime.php";

/**
 * Class context
 * @type("http://aps-standard.org/samples/sample-offering/context/1.0")
 * @implements("http://aps-standard.org/types/core/subscription/service/1.0")
 */
class context extends \APS\ResourceBase
{
    ## Strong relation (link) to the application instance
    /**
     * @link("http://aps-standard.org/samples/sample-offering/cloud/1.0")
     * @required
     */
    public $cloud;

    ## Weak relation (link) to collection of VPSes
    /**
     * @link("http://aps-standard.org/samples/sample-offering/vps/1.0[]")
     */
    public $vpses;
    /**
     * @link("http://aps-standard.org/samples/sample-offering/administration/1.0[]")
     */
    public $administration;
    /**
     * @link("http://aps-standard.org/samples/sample-offering/antivirus/1.0[]")
     */
    public $antiviruses;
    /**
     * @link("http://aps-standard.org/samples/sample-offering/ddosprotection/1.0[]")
     */
    public $ddosprotection;
    /**
     * @link("http://aps-standard.org/samples/sample-offering/webcms/1.0[]")
     */
    public $webcms;

    ## Resource counters
    /**
     * @type("http://aps-standard.org/types/core/resource/1.0#Counter")
     * @description("Total Disk Space Usage")
     * @unit("gb")
     */
    public $diskusagetotal;

    /**
     * @type("http://aps-standard.org/types/core/resource/1.0#Counter")
     * @description("Total CPU usage")
     * @unit("unit")
     */
    public $cpuusagetotal;

    /**
     * @type("http://aps-standard.org/types/core/resource/1.0#Counter")
     * @description("Total Memory usage")
     * @unit("mb")
     */
    public $memoryusagetotal;

    # This is the method that APS controller will call for getting resource usage from resource counters.
    # The method will collect resource usage by calling each VPS linked with this context.
    public function retrieve()
    {
        error_log("Fetching resource usage"); //For debugging only, see /var/log/httpd/ssl_error_log.
        ## Connect to the APS controller
        $apsc = \APS\Request::getController();
        ## Reset the local counters
        $disk = 0;
        $ram = 0;
        $cpu = 0;

        ## Collect resource usage from all VPSes
        foreach ($this->vpses as $vps) {
            $usage = $apsc->getIo()->sendRequest(\APS\Proto::GET,
                $apsc->getIo()->resourcePath($vps->aps->id, 'updateResourceUsage'));
            $usage = json_decode($usage);
            $disk = $disk + $usage->diskActualUsage;
            $ram = $ram + $usage->ramActualUsage;
            $cpu = $cpu + $usage->cpuActualUsage;
        }

        ## Update the APS resource counters
        $this->diskusagetotal->usage = $disk;
        $this->memoryusagetotal->usage = $ram;
        $this->cpuusagetotal->usage = $cpu;
    }
}
?>
