<?php

# logger class will be used for logging event notifications
require "logger.php";
require "aps/2/runtime.php";

/**
 * Class events
 * @type("http://aps-standard.org/samples/sample-offering/event/1.0")
 * @implements("http://aps-standard.org/types/core/resource/1.0")
 */
class event extends \APS\ResourceBase
{

    ## Require a link to the management context
    /**
     * @link("http://aps-standard.org/samples/sample-offering/context/1.0")
     * @required
     */
    public $context;

    ## Subscribe to Events during the provisioning process using PHP runtime
    public function provision()
    {

        #### Subscriptions to VPS related events:
        $subVPSavailable = new \APS\EventSubscription(\APS\EventSubscription::Available, "onVPSavailable");
        $subVPSremove = new \APS\EventSubscription(\APS\EventSubscription::Removed, "onVPSremove");
        $subVPSchange = new \APS\EventSubscription(\APS\EventSubscription::Changed, "onVPSchange");
        $subVPSofferLink = new \APS\EventSubscription(\APS\EventSubscription::Linked, "onVPSofferLink");
        $subVPSofferUnlink = new \APS\EventSubscription(\APS\EventSubscription::Unlinked, "onVPSofferUnlink");
        $subVPSuserLink = new \APS\EventSubscription(\APS\EventSubscription::Linked, "onVPSuserLink");
        $subVPSuserUnlink = new \APS\EventSubscription(\APS\EventSubscription::Unlinked, "onVPSuserUnlink");

        #### Subscriptions to creation and removal of customer�s domains:
        $subDomainAvailable = new \APS\EventSubscription(\APS\EventSubscription::Available, "onDomainAvailable");
        $subDomainRemove = new \APS\EventSubscription(\APS\EventSubscription::Removed, "onDomainRemove");

        #### Subscriptions to creation and removal of customer�s service users:
        $subUserAvailable = new \APS\EventSubscription(\APS\EventSubscription::Available, "onUserAvailable");
        $subUserRemove = new \APS\EventSubscription(\APS\EventSubscription::Removed, "onUserRemove");

        #### Connect to the APS controller
        $apsc = \APS\Request::getController();
        if ($apsc == null) {
            error_log("apsc is null");
        }

        #### Activate the subscriptions:
        $subVPSavailable->source->type = "http://aps-standard.org/samples/sample-offering/vps/1.0";
        $apsc->subscribe($this, $subVPSavailable);
        $subVPSremove->source->type = "http://aps-standard.org/samples/sample-offering/vps/1.0";
        $apsc->subscribe($this, $subVPSremove);
        $subVPSchange->source->type = "http://aps-standard.org/samples/sample-offering/vps/1.0";
        $apsc->subscribe($this, $subVPSchange);
        $subVPSofferLink->source->type = "http://aps-standard.org/samples/sample-offering/vps/1.0";
        $subVPSofferLink->relation = 'offer';
        $apsc->subscribe($this, $subVPSofferLink);
        $subVPSofferUnlink->source->type = "http://aps-standard.org/samples/sample-offering/vps/1.0";
        $subVPSofferUnlink->relation = 'offer';
        $apsc->subscribe($this, $subVPSofferUnlink);
        $subVPSuserLink->source->type = "http://aps-standard.org/samples/sample-offering/vps/1.0";
        $subVPSuserLink->relation = 'user';
        $apsc->subscribe($this, $subVPSuserLink);
        $subVPSuserUnlink->source->type = "http://aps-standard.org/samples/sample-offering/vps/1.0";
        $subVPSuserUnlink->relation = 'user';
        $apsc->subscribe($this, $subVPSuserUnlink);

        $subDomainAvailable->source->type = "http://parallels.com/aps/types/pa/dns/zone/1.0";
        $apsc->subscribe($this, $subDomainAvailable);
        $subDomainRemove->source->type = "http://parallels.com/aps/types/pa/dns/zone/1.0";
        $apsc->subscribe($this, $subDomainRemove);

        $subUserAvailable->source->type = "http://parallels.com/aps/types/pa/service-user/1.1";
        $apsc->subscribe($this, $subUserAvailable);
        $subUserRemove->source->type = "http://parallels.com/aps/types/pa/service-user/1.1";
        $apsc->subscribe($this, $subUserRemove);

    }

    ## For each type of Event Notifications, define a handler
    #### New VPS is available:
    /**
     * @verb(POST)
     * @path("/onVPSavailable")
     * @param("http://aps-standard.org/types/core/resource/1.0#Notification",body)
     */
    public function onVPSavailable($notification)
    {
        $this->logger("VPS created: " . json_format($notification));
    }

    #### A VPS is removed:
    /**
     * @verb(POST)
     * @path("/onVPSremove")
     * @param("http://aps-standard.org/types/core/resource/1.0#Notification",body)
     */
    public function onVPSremove($notification)
    {
        $this->logger("VPS removed: " . json_format($notification));
    }

    #### A VPS is linked or re-linked to an offer:
    /**
     * @verb(POST)
     * @path("/onVPSofferLink")
     * @param("http://aps-standard.org/types/core/resource/1.0#Notification",body)
     */
    public function onVPSofferLink($notification)
    {
        $this->logger("New offer Linked: " . json_format($notification));
    }

    #### A VPS is unlinked from its offer:
    /**
     * @verb(POST)
     * @path("/onVPSofferUnlink")
     * @param("http://aps-standard.org/types/core/resource/1.0#Notification",body)
     */
    public function onVPSofferUnlink($notification)
    {
        $this->logger("Offer Unlinked: " . json_format($notification));
    }

    #### A VPS is linked or re-linked to a service user:
    /**
     * @verb(POST)
     * @path("/onVPSuserLink")
     * @param("http://aps-standard.org/types/core/resource/1.0#Notification",body)
     */
    public function onVPSuserLink($notification)
    {
        $this->logger("New user Linked: " . json_format($notification));
    }

    #### A VPS is unlinked from its service user:
    /**
     * @verb(POST)
     * @path("/onVPSuserUnlink")
     * @param("http://aps-standard.org/types/core/resource/1.0#Notification",body)
     */
    public function onVPSuserUnlink($notification)
    {
        $this->logger("User UnLinked: " . json_format($notification));
    }

    #### VPS properties are changed:
    /**
     * @verb(POST)
     * @path("/onVPSchange")
     * @param("http://aps-standard.org/types/core/resource/1.0#Notification",body)
     */
    public function onVPSchange($notification)
    {
        $this->logger("VPS properties changed: " . json_format($notification));
    }

    #### New hosted domain is available:
    /**
     * @verb(POST)
     * @path("/onDomainAvailable")
     * @param("http://aps-standard.org/types/core/resource/1.0#Notification",body)
     */
    public function onDomainAvailable($notification)
    {
        $this->logger("New Domain available: " . json_format($notification));
    }

    #### A hosted domain is removed:
    /**
     * @verb(POST)
     * @path("/onDomainRemove")
     * @param("http://aps-standard.org/types/core/resource/1.0#Notification",body)
     */
    public function onDomainRemove($notification)
    {
        $this->logger("Domain removed: " . json_format($notification));
    }

    #### New service user is available:
    /**
     * @verb(POST)
     * @path("/onUserAvailable")
     * @param("http://aps-standard.org/types/core/resource/1.0#Notification",body)
     */
    public function onUserAvailable($notification)
    {
        $this->logger("New User available: " . json_format($notification));
    }

    #### A service user is removed:
    /**
     * @verb(POST)
     * @path("/onUserRemove")
     * @param("http://aps-standard.org/types/core/resource/1.0#Notification",body)
     */
    public function onUserRemove($notification)
    {
        $this->logger("User removed: " . json_format($notification));
    }

    ## Define a function that returns the list of Notifications logged by the service
    /**
     * @verb(GET)
     * @path("/readNotifications")
     * @return(string)
     */
    public function readNotifications()
    {
        $log = new Logging();
        $log->logfile('./' . $this->aps->id . '.log');
        return $log->flushlog();
    }

    ## Define a Logger function that will log a new Event Notification as a message
    function logger($message)
    {
        $requester = $_SERVER['REMOTE_ADDR'];
        $log = new Logging();
        $log->logfile('./' . $this->aps->id . '.log');
        $log->logwrite($requester . ":" . $message);
        $log->logclose();
    }
}
?>
