define([
	"dojo/_base/declare",
	"dojo/when",

	"aps/ResourceStore",
	"aps/_View",

	"./displayError.js"
], function(
	declare,
	when,

	Store,
	_View,

	displayError
) {
	var page, grid;

	return declare(_View, {
		init: function() {
			/* Create the data store */
			var store = new Store({
				apsType: "http://aps-standard.org/samples/sample-offering/vps/1.0",
				target: "/aps/2/resources/"
			});

			/* Widget processing logic */
			var refresh = function() {
				grid.refresh();
			};

			function changeState(state, btn) {
				var sel = grid.get("selectionArray");
				var counter = sel.length;

				/* Clear the current messages on the screen */
				page.get("messageList").removeAll();

				sel.forEach(function(vpsId) {
					console.log("I'm trying to change state of VPS with id = [" + vpsId + "]");

					var vps = {
						aps: {
							id: vpsId
						},
						state: state
					};

					/* Save the VPS properties */
					when(store.put(vps),
						/* If success, process the next VPS until the list is empty */
						function() {
							console.log("State of VPS with id = [" + vpsId + "] changed");
							sel.splice(sel.indexOf(vpsId), 1);
							grid.refresh();
							if (--counter) {
								btn.cancel();
							} /* Remove busy state for button */
						},
						/* If failure, call the error handler */
						function(e) {
							displayError(e);
							if (--counter) {
								btn.cancel();
							}
						}
					);
				});
			}

			/* Create the on-click handler for the *Stop* button */
			var stop = function() {
				changeState("Stopped", this);
			};

			/* Create the on-click handler for the *Start* button */
			var start = function() {
				changeState("Running", this);
			};

			/* Define and return widgets */
			return ["aps/Grid", {
				id: this.genId("mysrv_grid"),
				store: store,
				selectionMode: "multiple",
				columns: [{
					field: "name",
					name: "Name"
				}, {
					field: "hardware.memory",
					name: "RAM",
					title: "RAM"
				}, {
					field: "hardware.diskspace",
					name: "Disk Space",
					title: "Disk"
				}, {
					field: "hardware.CPU.number",
					name: "CPU"
				}, {
					field: "platform.OS.name",
					name: "OS"
				}, {
					field: "state",
					name: "State"
				}, {
					field: "aps.status",
					name: "APS Status"
				}]
			}, [
				["aps/Toolbar", [
					["aps/ToolbarButton", {
						id: this.genId("mysrv_start"),
						iconClass: "fa-play",
						type: "success",
						label: "Start",
						requireItems: true,
						onClick: start
					}],
					["aps/ToolbarButton", {
						id: this.genId("mysrv_stop"),
						iconClass: "fa-pause",
						type: "warning",
						label: "Stop",
						requireItems: true,
						onClick: stop
					}],
					["aps/ToolbarButton", {
						id: this.genId("mysrv_refresh"),
						iconClass: "fa-refresh",
						label: "Refresh",
						autoBusy: false,
						onClick: refresh
					}]
				]]
			]];
		}, // End of Init
		onContext: function() {
			grid = grid || this.byId("mysrv_grid");
			page = page || this.byId("apsPageContainer");
			grid.refresh();
			aps.apsc.hideLoading();
		}
	}); // End of Declare
}); // End of Define