define([
    "dojo/_base/declare",
    "dojo/when",
    "dojo/promise/all",
    "dojox/mvc/getStateful",
    "dojox/mvc/at",

    "aps/_View",
    "aps/ResourceStore",
    "aps/Memory",

    "dojo/text!./../newvps.json",
    "dojo/text!./../newoffer.json"
], function (declare,
             when,
             all,
             getStateful,
             at,

             _View,
             Store,
             Memory,

             newVPS,
             newOffer) {
    return declare(_View, {
        init: function () {
            var self = this;

            /* Declare and define data sources */
            this.offerCache = new Memory({
                data: [],
                idProperty: "aps.id"
            }); // Cache with a list of offers

            this.offerCtrl = getStateful({
                model: JSON.parse(newOffer)
            }); // Stores current offer

            this.vpsModel = getStateful({
                newVPS: JSON.parse(newVPS)
            });

            /* A list of all service users */
            this.userSelectionList = new Memory({
                data: [],
                idProperty: "aps.id"
            });

            /* Handler on offer change */
            function onChangeOffer(offerID) {
                if (offerID) {
                    when(self.offerCache.get(offerID), function (data) {
                        /* For debugging only */
                        console.log(offerID);
                        self.offerCtrl.set("model", data);
                        self.vpsModel.newVPS.offer.aps.set("id", data.aps.id);
                    });
                }
            }

            /* Create widgets */
            return [
                ["aps/Panel", {id: this.genId("wizardNew_panel1"), title: _("Server")}, [
                    ["aps/FieldSet", {id: this.genId("srvNew1_general")}, [
                        ["aps/TextBox", {
                            id: this.genId("srvNew1_serverName"),
                            gridSize: 'md-3 xs-12',
                            label: _("Server Name"),
                            value: at(this.vpsModel.newVPS, "name"),
                            required: true
                        }],
                        ["aps/TextBox", {
                            id: this.genId("srvNew1_description"),
                            gridSize: 'md-3 xs-12',
                            label: _("Description"),
                            value: at(this.vpsModel.newVPS, "description")
                        }]
                    ]]
                ]],
                ["aps/Panel", {id: this.genId("wizardNew_panel2"), title: _("Owner")}, [
                    ["aps/FieldSet", {id: this.genId("srvNew1_owner")}, [
                        ["aps/Select", {
                            id: this.genId("srvNew1_userSelect"),
                            gridSize: 'md-3 xs-12',
                            required: true,
                            label: _("User Name"),
                            labelAttr: "fullName",
                            value: at(this.vpsModel.newVPS.user.aps, "id")
                        }]
                    ]]
                ]],
                ["aps/Panel", {id: this.genId("wizardNew_panel3"), title: _("Offer")}, [
                    ["aps/FieldSet", {id: this.genId("srvNew1_offers")}, [
                        ["aps/Select", {
                            id: this.genId("srvNew1_offerSelect"),
                            gridSize: 'md-3 xs-12',
                            required: true,
                            label: _("Bundle"),
                            onChange: onChangeOffer,
                            labelAttr: "profileName",
                            value: at(this.vpsModel.newVPS.offer.aps, "id")
                        }]
                    ]]
                ]],
                ["aps/Panel", {id: this.genId("wizardNew_panel4"), title: _("Configuration")}, [
                    ["aps/FieldSet", {id: this.genId("srvNew1_props")}, [
                        ["aps/Select", {
                            id: this.genId("srvNew1_os"),
                            gridSize: 'md-3 xs-12',
                            label: _("OS"),
                            disabled: true,
                            required: true,
                            value: at(this.offerCtrl.model.platform.OS, "name")
                        }],
                        ["aps/Spinner", {
                            id: this.genId("srvNew1_cpu"),
                            gridSize: 'md-3 xs-12',
                            label: _("CPU Cores"),
                            minimum: 1,
                            maximum: at(this.offerCtrl.model.hardware.CPU, "number"),
                            value: at(this.vpsModel.newVPS.hardware.CPU, "number"),
                            step: 1
                        }],
                        ["aps/Spinner", {
                            id: this.genId("srvNew1_disk"),
                            gridSize: 'md-3 xs-12',
                            label: _("Disk Space"),
                            minimum: 1,
                            maximum: at(this.offerCtrl.model.hardware, "diskspace"),
                            value: at(this.vpsModel.newVPS.hardware, "diskspace"),
                            legend: _("GB"),
                            step: 1
                        }],
                        ["aps/Spinner", {
                            id: this.genId("srvNew1_ram"),
                            gridSize: 'md-3 xs-12',
                            label: _("RAM"),
                            minimum: 128,
                            maximum: at(this.offerCtrl.model.hardware, "memory"),
                            value: at(this.vpsModel.newVPS.hardware, "memory"),
                            step: 128,
                            legend: _("MB")
                        }]
                    ]]
                ]]
            ];
        }, // End of Init

        onShow: function() {
            if (aps.app.isNew) {
                this.vpsModel.set("newVPS", JSON.parse(newVPS));
            }
        },

        onContext: function () {
            var self = this;

            var offerStore = new Store({ // The list of offers for selection
                apsType: "http://aps-standard.org/samples/sample-offering/offer/1.0",
                target: "/aps/2/resources/"
            });

            var userStore = new Store({ // The list of users for selection
                apsType: "http://aps-standard.org/types/core/user/1.0",
                target: "/aps/2/resources/"
            });

            all([offerStore.query(), userStore.query()]).then(function (offersAndUsers) {
                var offers = offersAndUsers[0];

                if (offers.length === 0) {
                    return;
                }

                var users = offersAndUsers[1];

                self.userSelectionList.setData(users);
                self.offerCache.setData(offers);
                self.offerCtrl.set("model", offers[0]);

                self.vpsModel.newVPS.offer.aps.set("id", self.offerCtrl.model.aps.id);
                self.byId("srvNew1_userSelect").set("store", self.userSelectionList);
                self.byId("srvNew1_offerSelect").set("store", self.offerCache);
            }).then(function () {
                aps.apsc.hideLoading();
            });
        },

        /* Create handlers for the navigation buttons */
        onNext: function () {
            if (!this.userSelectionList.data.length) {
                aps.apsc.cancelProcessing();
                return;
            }

            var page = this.byId("apsPageContainer");
            page.get("messageList").removeAll();
            if (!page.validate()) {
                aps.apsc.cancelProcessing();
                return;
            }

            /* Proceed to the next wizard view */
            aps.app.isNew = false;
            this.vpsModel.newVPS.set("userName", this.userSelectionList.get(this.vpsModel.newVPS.user.aps.id).fullName);
            this.vpsModel.newVPS.platform.OS.set("name", this.byId("srvNew1_os").value);
            aps.apsc.next(this.vpsModel.newVPS);
        },

        onCancel: function () {
            aps.apsc.gotoView("servers");
        }
    }); // End of Declare
}); // End of Define
